@@ -1,12 +1,13 @@
-from .db import db
+
+import datetime
 from hashlib import sha256
-import shutil
 import os
 import json
 
 import pandas as pd
 from flask import current_app as app
 
+from .db import db
 from .hit import HIT
 
 
@@ -19,22 +20,23 @@ class Project(db.Model):
     email = db.Column(db.String)
     hits = db.relationship("HIT", backref="project", cascade="all, delete-orphan")
 
+    created_at = db.Column(db.DateTime, default=datetime.datetime.now)
+    updated_at = db.Column(db.DateTime, onupdate=datetime.datetime.now)
+
     def __init__(self, id, name, email, hits, **kwargs):
         hashstr = Project.get_hashtr(id)
         self.id = sha256(hashstr.encode()).digest()
-        self.update(id, name, email, hits)
-
-    def update(self, id, name, email, hits, **kwargs):
         self.name = name
         self.email = email
 
         hashstr = Project.get_hashtr(id)
         for hit_dict in hits:
-            hit = db.session.query(HIT).get(HIT.get_id(hashstr, hit_dict['id']))
-            if hit is None:
-                self.hits.append(HIT(**hit_dict, hashstr=hashstr))
-            else:
-                hit.update(**hit_dict, hashstr=hashstr)
+            hit = HIT(**hit_dict, project_id=id)
+            self.hits.append(hit)
+
+            repeats = hit_dict.get('repeat', 1)
+            for r in range(repeats):
+                hit.instantiate()
 
     def get_dataframe(self):
         list_of_instances = list()
@@ -51,13 +53,12 @@ class Project(db.Model):
         
         return df
 
-    def as_dict(self, with_hits=False, with_instances=False):
+    def as_dict(self, with_hits=False, with_instances=False, with_config=False):
         project_dict = {c.name: getattr(self, c.name)
                         for c in self.__table__.columns}
         project_dict['id'] = project_dict['id'].hex()
         if with_hits:
-            project_dict['hits'] = [hit.as_dict(
-                with_instances=with_instances) for hit in self.hits]
+            project_dict['hits'] = [hit.as_dict(with_instances=with_instances, with_config=with_config) for hit in self.hits]
         return project_dict
 
     def info(self):
@@ -90,20 +91,9 @@ class Project(db.Model):
 
         return Project(**proj_dict)
 
-    def make_download(self, csv=False):
-        # create a folder to store all the files
-        dirpath = os.path.join(app.config['TMP_PATH'], self.id.hex())
-        if os.path.exists(dirpath) and os.path.isdir(dirpath):
-            shutil.rmtree(dirpath)
-
-        os.mkdir(dirpath)
-
-        # go over all hit instances in the project
-        total_files = 0
+    def stream_download(self, z, base_path, submitted_only=True, csv=False):
         for hit in self.hits:
             for instance in hit.instances:
-                print(instance.id.hex())
-                instance_path, num_files = instance.make_download(base_dir=dirpath, csv=csv)
-                total_files += num_files
-
-        return dirpath, total_files
+                if submitted_only and not instance.submitted:
+                    continue
+                yield from instance.stream_download(z, os.path.join(base_path, instance.id.hex()), csv=csv)
